set(SYNVIEW_VERSION "1.3.12.0")


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was SynViewConfig.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

if (NOT TARGET SynView::synview)
    if (NOT UNIX AND NOT WIN32)
        if (NOT SynView_FIND_QUIETLY)
            message("Find script only implemented for Linux and Winows target systems, but the target system is ${CMAKE_SYSTEM_NAME}")
        endif()
        set(SynView_FOUND False)
    else()
        if (UNIX)
            set(SYNVIEW_ROOT "/opt/synview")
        else()
            set(SYNVIEW_ROOT "")
        endif()
        
        set_and_check(SV_INCLUDE_DIR "${PACKAGE_PREFIX_DIR}/opt/synview/include")
        set_and_check(SV_LIBS_DIR "${PACKAGE_PREFIX_DIR}/opt/synview/lib")

        if (WIN32)
            set_and_check(SV_DLL_DIR "${PACKAGE_PREFIX_DIR}/opt/synview/bin")
            set(SV_BINARY_INFIX "")
            if (CMAKE_SIZEOF_VOID_P EQUAL 8)
                set(SV_BINARY_INFIX ".x64")
            endif()
        endif()

        # check header existence
       foreach(_HEADER IN ITEMS
            sv.synview.h
            sv.synview.class.cpp
            sv.synview.class.h
            sv.synview.defs.h
            sv.synview.enums.h
            sv.synview.imgproc.h
            sv.synview.ini.h
            sv.synview.status.h
        )
            if (NOT EXISTS "${SV_INCLUDE_DIR}/${_HEADER}")
                message(FATAL_ERROR "Required Header not found ${SV_INCLUDE_DIR}/${_HEADER}")
            endif()
        endforeach()

        macro(sv_import_shared_lib LIB_NAME LIB_FILE)
            if (UNIX)
                add_library("SynView::${LIB_NAME}" SHARED IMPORTED GLOBAL)
                set_and_check(SO_FILE "${SV_LIBS_DIR}/lib${LIB_FILE}.so")
                
                set_target_properties("SynView::${LIB_NAME}" PROPERTIES
                    IMPORTED_LOCATION ${SO_FILE}
                )
            else() # WIN32
                set_and_check(DLL_FILE "${SV_DLL_DIR}/${LIB_FILE}${SV_BINARY_INFIX}.dll")
                set_and_check(LIB_FILE "${SV_LIBS_DIR}/${LIB_FILE}${SV_BINARY_INFIX}.lib")

                add_library("SynView::${LIB_NAME}" SHARED IMPORTED GLOBAL)
                set_target_properties("SynView::${LIB_NAME}" PROPERTIES
                    IMPORTED_LOCATION ${DLL_FILE}
                    IMPORTED_IMPLIB ${LIB_FILE}
                )
            endif()

            target_include_directories("SynView::${LIB_NAME}" INTERFACE ${SV_INCLUDE_DIR})
        endmacro()

        sv_import_shared_lib(synview sv.synview) # synview C consumer
        sv_import_shared_lib(ini sv.synview.ini) # ini file access
        
        if (WIN32)
            # for some reason we use a different name for the import lib on windows
            set_and_check(DLL_FILE "${SV_DLL_DIR}/sv.imgproc${SV_BINARY_INFIX}.dll")
            set_and_check(LIB_FILE "${SV_LIBS_DIR}/sv.synview.imgproc${SV_BINARY_INFIX}.lib")
            
            add_library(SynView::imgproc SHARED IMPORTED GLOBAL)
            set_target_properties("SynView::imgproc" PROPERTIES
                IMPORTED_LOCATION ${DLL_FILE}
                IMPORTED_IMPLIB ${LIB_FILE}
            )
        else()
            sv_import_shared_lib(imgproc sv.imgproc) # image transformation
        endif()

        set_and_check(SYNVIEW_CTI_DIR "${PACKAGE_PREFIX_DIR}/opt/synview/cti64")
        if(UNIX)
            set_and_check(SO_FILE "${SYNVIEW_CTI_DIR}/libsv.gentl.cti")

            add_library(SynView::gentl MODULE IMPORTED GLOBAL)

            sv_import_shared_lib(greg sv.greg)
            sv_import_shared_lib(jpeg sv.jpeg)
            sv_import_shared_lib(log sv.log)
            sv_import_shared_lib(osdep sv.osdep)
        
            add_library(SynView::gentl-shared SHARED IMPORTED GLOBAL)
            target_link_libraries(SynView::gentl-shared
                INTERFACE
                    SynView::greg
                    SynView::jpeg
                    SynView::log
                    SynView::osdep
            )
            
            set_target_properties(SynView::gentl SynView::gentl-shared PROPERTIES
                IMPORTED_LOCATION ${SO_FILE}
            )
        else() # WIN32
            set_and_check(DLL_FILE "${SYNVIEW_CTI_DIR}/sv.gentl${SV_BINARY_INFIX}.cti")
            add_library(SynView::gentl MODULE IMPORTED GLOBAL)
            
            set_target_properties(SynView::gentl PROPERTIES
                IMPORTED_LOCATION ${DLL_FILE}
            )
        endif()

        add_library(SynView::cpp INTERFACE IMPORTED GLOBAL)
        target_sources(SynView::cpp INTERFACE
            "${SV_INCLUDE_DIR}/sv.synview.class.cpp"
            "${SV_INCLUDE_DIR}/sv.synview.class.h"
        )
        target_include_directories(SynView::cpp INTERFACE ${SV_INCLUDE_DIR})
        target_link_libraries(SynView::cpp INTERFACE SynView::synview)
    endif()

    if(SynView_FOUND)
        set(SYNVIEW_VERSION ${SYNVIEW_VERSION} CACHE STRING "SynView Version" FORCE)
        set(SYNVIEW_CTI_DIR ${SYNVIEW_CTI_DIR} CACHE PATH "Directory containing the installed synview cti file" FORCE)
        if (WIN32)
            set(SYNVIEW_BINARY_DIR "${PACKAGE_PREFIX_DIR}/opt/synview/bin/bin" CACHE PATH "Directory containing the installed synview dll files" FORCE)
        endif()
        if("0" EQUAL 0)
            set(SYNVIEW_OFFICIAL_RELEASE True CACHE BOOL "Is this the synview version an official release version?" FORCE)
        else()
            st(SYNVIEW_OFFICIAL_RELEASE False CACHE BOOL "Is this the synview version an official release version?" FORCE)
        endif()
    endif()
endif()

check_required_components(SynView)
